"""
Test the performance of pathlib operations.

This benchmark stresses the creation of small objects, globbing, and system
calls.
"""

# Python imports
import os
import pathlib
import shutil
import tempfile

import pyperf


NUM_FILES = 2000


def generate_filenames(tmp_path, num_files):
    i = 0
    while num_files:
        for ext in [".py", ".txt", ".tar.gz", ""]:
            i += 1
            yield os.path.join(tmp_path, str(i) + ext)
            num_files -= 1


def setup(num_files):
    tmp_path = tempfile.mkdtemp()
    for fn in generate_filenames(tmp_path, num_files):
        with open(fn, "wb") as f:
            f.write(b'benchmark')

    return tmp_path


def bench_pathlib(loops, tmp_path):
    base_path = pathlib.Path(tmp_path)

    # Warm up the filesystem cache and keep some objects in memory.
    path_objects = list(base_path.iterdir())
    # FIXME: does this code really cache anything?
    for p in path_objects:
        p.stat()
    assert len(path_objects) == NUM_FILES, len(path_objects)

    range_it = range(loops)
    t0 = pyperf.perf_counter()

    for _ in range_it:
        # Do something simple with each path.
        for p in base_path.iterdir():
            p.stat()
        for p in base_path.glob("*.py"):
            p.stat()
        for p in base_path.iterdir():
            p.stat()
        for p in base_path.glob("*.py"):
            p.stat()

    return pyperf.perf_counter() - t0


if __name__ == "__main__":
    runner = pyperf.Runner()
    runner.metadata['description'] = ("Test the performance of "
                                      "pathlib operations.")

    modname = pathlib.__name__
    runner.metadata['pathlib_module'] = modname

    tmp_path = setup(NUM_FILES)
    try:
        runner.bench_time_func('pathlib', bench_pathlib, tmp_path)
    finally:
        shutil.rmtree(tmp_path)



"""Test the performance of the Django template system.

This will have Django generate a 150x150-cell HTML table.
"""

import pyperf

import django.conf
from django.template import Context, Template


# 2016-10-10: Python 3.6 takes 380 ms
DEFAULT_SIZE = 100


def bench_django_template(runner, size):
    template = Template("""<table>
{% for row in table %}
<tr>{% for col in row %}<td>{{ col|escape }}</td>{% endfor %}</tr>
{% endfor %}
</table>
    """)
    table = [range(size) for _ in range(size)]
    context = Context({"table": table})

    runner.bench_func('django_template', template.render, context)


def prepare_cmd(runner, cmd):
    cmd.append("--table-size=%s" % runner.args.table_size)


if __name__ == "__main__":
    django.conf.settings.configure(TEMPLATES=[{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
    }])
    django.setup()

    runner = pyperf.Runner()
    cmd = runner.argparser
    cmd.add_argument("--table-size",
                     type=int, default=DEFAULT_SIZE,
                     help="Size of the HTML table, height and width "
                          "(default: %s)" % DEFAULT_SIZE)

    args = runner.parse_args()
    runner.metadata['description'] = "Django template"
    runner.metadata['django_version'] = django.__version__
    runner.metadata['django_table_size'] = args.table_size

    bench_django_template(runner, args.table_size)



"""
The Computer Language Benchmarks Game
http://benchmarksgame.alioth.debian.org/

Contributed by Sokolov Yura, modified by Tupteq.
"""

import pyperf


DEFAULT_ARG = 9


def fannkuch(n):
    count = list(range(1, n + 1))
    max_flips = 0
    m = n - 1
    r = n
    perm1 = list(range(n))
    perm = list(range(n))
    perm1_ins = perm1.insert
    perm1_pop = perm1.pop

    while 1:
        while r != 1:
            count[r - 1] = r
            r -= 1

        if perm1[0] != 0 and perm1[m] != m:
            perm = perm1[:]
            flips_count = 0
            k = perm[0]
            while k:
                perm[:k + 1] = perm[k::-1]
                flips_count += 1
                k = perm[0]

            if flips_count > max_flips:
                max_flips = flips_count

        while r != n:
            perm1_ins(r, perm1_pop(0))
            count[r] -= 1
            if count[r] > 0:
                break
            r += 1
        else:
            return max_flips


if __name__ == "__main__":
    runner = pyperf.Runner()
    arg = DEFAULT_ARG
    runner.bench_func('fannkuch', fannkuch, arg)



"""
Benchmark for asyncio websocket server and client performance
transferring 1MB of data.

Author: Kumar Aditya
"""

import pyperf
import websockets.server
import websockets.client
import websockets.exceptions
import asyncio

CHUNK_SIZE = 1024 ** 2
DATA = b"x" * CHUNK_SIZE

stop: asyncio.Event


async def handler(websocket) -> None:
    for _ in range(100):
        await websocket.recv()

    stop.set()


async def send(ws):
    try:
        await ws.send(DATA)
    except websockets.exceptions.ConnectionClosedOK:
        pass


async def main() -> None:
    global stop
    t0 = pyperf.perf_counter()
    stop = asyncio.Event()
    async with websockets.server.serve(handler, "", 8001):
        async with websockets.client.connect("ws://localhost:8001") as ws:
            await asyncio.gather(*[send(ws) for _ in range(100)])
        await stop.wait()
    return pyperf.perf_counter() - t0


if __name__ == "__main__":
    runner = pyperf.Runner()
    runner.metadata['description'] = "Benchmark asyncio websockets"
    runner.bench_async_func('asyncio_websockets', main)



"""
Build a subset of Python's documentation using Sphinx
"""

import io
import os
from pathlib import Path
import shutil

import pyperf
from sphinx.cmd.build import main as sphinx_main


# Sphinx performs a lot of filesystem I/O when it operates. This can cause the
# results to be highly variable. Instead, we pre-load all of the source files
# and then monkeypatch "open" so that Sphinx is reading from in-memory
# `io.BytesIO` and `io.StringIO` objects.


DOC_ROOT = (Path(__file__).parent / "data" / "Doc").resolve()


_orig_open = open


preloaded_files = {}


def read_all_files():
    for filename in DOC_ROOT.glob("**/*"):
        if filename.is_file():
            preloaded_files[str(filename)] = filename.read_bytes()


def open(
    file,
    mode="r",
    buffering=-1,
    encoding=None,
    errors=None,
    newline=None,
    closefd=True,
    opener=None,
):
    if isinstance(file, Path):
        file = str(file)

    if isinstance(file, str):
        if "r" in mode and file in preloaded_files:
            if "b" in mode:
                return io.BytesIO(preloaded_files[file])
            else:
                return io.StringIO(preloaded_files[file].decode(encoding or "utf-8"))
        elif "w" in mode and DOC_ROOT in Path(file).parents:
            if "b" in mode:
                newfile = io.BytesIO()
            else:
                newfile = io.StringIO()
            preloaded_files[file] = newfile
            return newfile

    return _orig_open(
        file,
        mode=mode,
        buffering=buffering,
        encoding=encoding,
        errors=errors,
        newline=newline,
        closefd=closefd,
        opener=opener,
    )


__builtins__.open = open


def replace(src, dst):
    pass


os.replace = replace


def build_doc(doc_root):
    # Make sure there is no caching going on
    t0 = pyperf.perf_counter()
    sphinx_main(
        [
            "--builder",
            "dummy",
            "--doctree-dir",
            str(doc_root / "build" / "doctrees"),
            "--jobs",
            "1",
            "--silent",
            "--fresh-env",
            "--write-all",
            str(doc_root),
            str(doc_root / "build" / "html"),
        ]
    )
    return pyperf.perf_counter() - t0


def bench_sphinx(loops, doc_root):
    if (DOC_ROOT / "build").is_dir():
        shutil.rmtree(DOC_ROOT / "build")
    read_all_files()

    runs_total = 0
    for _ in range(loops):
        runs_total += build_doc(doc_root)
        if (DOC_ROOT / "build").is_dir():
            shutil.rmtree(DOC_ROOT / "build")

    return runs_total


if __name__ == "__main__":
    runner = pyperf.Runner()

    runner.metadata["description"] = (
        "Render documentation with Sphinx, like the CPython docs"
    )
    args = runner.parse_args()

    runner.bench_time_func("sphinx", bench_sphinx, DOC_ROOT)



#!/usr/bin/env python
"""
Pure-Python Implementation of the AES block-cipher.

Benchmark AES in CTR mode using the pyaes module.
"""

import pyperf

import pyaes

# 23,000 bytes
CLEARTEXT = b"This is a test. What could possibly go wrong? " * 500

# 128-bit key (16 bytes)
KEY = b'\xa1\xf6%\x8c\x87}_\xcd\x89dHE8\xbf\xc9,'


def bench_pyaes(loops):
    range_it = range(loops)
    t0 = pyperf.perf_counter()

    for loops in range_it:
        aes = pyaes.AESModeOfOperationCTR(KEY)
        ciphertext = aes.encrypt(CLEARTEXT)

        # need to reset IV for decryption
        aes = pyaes.AESModeOfOperationCTR(KEY)
        plaintext = aes.decrypt(ciphertext)

        # explicitly destroy the pyaes object
        aes = None

    dt = pyperf.perf_counter() - t0
    if plaintext != CLEARTEXT:
        raise Exception("decrypt error!")

    return dt


if __name__ == "__main__":
    runner = pyperf.Runner()
    runner.metadata['description'] = ("Pure-Python Implementation "
                                      "of the AES block-cipher")
    runner.bench_time_func('crypto_pyaes', bench_pyaes)



import pyperf

from sqlglot import parse_one, transpile
from sqlglot.optimizer import optimize, normalize


SQL = """
select
        supp_nation,
        cust_nation,
        l_year,
        sum(volume) as revenue
from
        (
                select
                        n1.n_name as supp_nation,
                        n2.n_name as cust_nation,
                        extract(year from l_shipdate) as l_year,
                        l_extendedprice * (1 - l_discount) as volume
                from
                        supplier,
                        lineitem,
                        orders,
                        customer,
                        nation n1,
                        nation n2
                where
                        s_suppkey = l_suppkey
                        and o_orderkey = l_orderkey
                        and c_custkey = o_custkey
                        and s_nationkey = n1.n_nationkey
                        and c_nationkey = n2.n_nationkey
                        and (
                                (n1.n_name = 'FRANCE' and n2.n_name = 'GERMANY')
                                or (n1.n_name = 'GERMANY' and n2.n_name = 'FRANCE')
                        )
                        and l_shipdate between date '1995-01-01' and date '1996-12-31'
        ) as shipping
group by
        supp_nation,
        cust_nation,
        l_year
order by
        supp_nation,
        cust_nation,
        l_year;
"""

TPCH_SCHEMA = {
    "lineitem": {
        "l_orderkey": "uint64",
        "l_partkey": "uint64",
        "l_suppkey": "uint64",
        "l_linenumber": "uint64",
        "l_quantity": "float64",
        "l_extendedprice": "float64",
        "l_discount": "float64",
        "l_tax": "float64",
        "l_returnflag": "string",
        "l_linestatus": "string",
        "l_shipdate": "date32",
        "l_commitdate": "date32",
        "l_receiptdate": "date32",
        "l_shipinstruct": "string",
        "l_shipmode": "string",
        "l_comment": "string",
    },
    "orders": {
        "o_orderkey": "uint64",
        "o_custkey": "uint64",
        "o_orderstatus": "string",
        "o_totalprice": "float64",
        "o_orderdate": "date32",
        "o_orderpriority": "string",
        "o_clerk": "string",
        "o_shippriority": "int32",
        "o_comment": "string",
    },
    "customer": {
        "c_custkey": "uint64",
        "c_name": "string",
        "c_address": "string",
        "c_nationkey": "uint64",
        "c_phone": "string",
        "c_acctbal": "float64",
        "c_mktsegment": "string",
        "c_comment": "string",
    },
    "part": {
        "p_partkey": "uint64",
        "p_name": "string",
        "p_mfgr": "string",
        "p_brand": "string",
        "p_type": "string",
        "p_size": "int32",
        "p_container": "string",
        "p_retailprice": "float64",
        "p_comment": "string",
    },
    "supplier": {
        "s_suppkey": "uint64",
        "s_name": "string",
        "s_address": "string",
        "s_nationkey": "uint64",
        "s_phone": "string",
        "s_acctbal": "float64",
        "s_comment": "string",
    },
    "partsupp": {
        "ps_partkey": "uint64",
        "ps_suppkey": "uint64",
        "ps_availqty": "int32",
        "ps_supplycost": "float64",
        "ps_comment": "string",
    },
    "nation": {
        "n_nationkey": "uint64",
        "n_name": "string",
        "n_regionkey": "uint64",
        "n_comment": "string",
    },
    "region": {
        "r_regionkey": "uint64",
        "r_name": "string",
        "r_comment": "string",
    },
}


def bench_parse(loops):
    elapsed = 0
    for _ in range(loops):
        t0 = pyperf.perf_counter()
        parse_one(SQL)
        elapsed += pyperf.perf_counter() - t0
    return elapsed


def bench_transpile(loops):
    elapsed = 0
    for _ in range(loops):
        t0 = pyperf.perf_counter()
        transpile(SQL, write="spark")
        elapsed += pyperf.perf_counter() - t0
    return elapsed


def bench_optimize(loops):
    elapsed = 0
    for _ in range(loops):
        t0 = pyperf.perf_counter()
        optimize(parse_one(SQL), TPCH_SCHEMA)
        elapsed += pyperf.perf_counter() - t0
    return elapsed


def bench_normalize(loops):
    elapsed = 0
    for _ in range(loops):
        conjunction = parse_one("(A AND B) OR (C AND D) OR (E AND F) OR (G AND H)")
        t0 = pyperf.perf_counter()
        normalize.normalize(conjunction)
        elapsed += pyperf.perf_counter() - t0
    return elapsed


BENCHMARKS = {
    "parse": bench_parse,
    "transpile": bench_transpile,
    "optimize": bench_optimize,
    "normalize": bench_normalize
}


def add_cmdline_args(cmd, args):
    cmd.append(args.benchmark)


def add_parser_args(parser):
    parser.add_argument(
        "benchmark",
        choices=BENCHMARKS,
        help="Which benchmark to run."
    )


if __name__ == "__main__":
    runner = pyperf.Runner(add_cmdline_args=add_cmdline_args)
    runner.metadata['description'] = "SQLGlot V2 benchmark"
    add_parser_args(runner.argparser)
    args = runner.parse_args()
    benchmark = args.benchmark

    runner.bench_time_func(f"sqlglot_v2_{benchmark}", BENCHMARKS[benchmark])



# Generic helpers for working with a Python executable.

import hashlib

from pyperformance import _pythoninfo


def get_id(python=None, prefix=None, *, short=True):
    """Return a string that uniquely identifies the given Python executable."""
    if isinstance(python, str):
        python = _pythoninfo.get_info(python)

    data = [
        # "executable" represents the install location
        # (and build, to an extent).
        python.sys.executable,
        # sys.version encodes version, git info, build_date, and build_tool.
        python.sys.version,
        python.sys.implementation.name.lower(),
        ".".join(str(v) for v in python.sys.implementation.version),
        str(python.sys.api_version),
        python.pyc_magic_number.hex(),
    ]
    # XXX Add git info if a dev build.

    h = hashlib.sha256()
    for value in data:
        h.update(value.encode("utf-8"))
    # XXX Also include the sorted output of "python -m pip freeze"?
    py_id = h.hexdigest()
    if short:
        py_id = py_id[:12]

    if prefix:
        if prefix is True:
            major, minor = python.sys.version_info[:2]
            py_id = f"{python.sys.implementation.name}{major}.{minor}-{py_id}"
        else:
            py_id = prefix + py_id

    return py_id



"""Test the performance of pprint.PrettyPrinter.

This benchmark was available as `python -m pprint` until Python 3.12.

Authors: Fred Drake (original), Oleg Iarygin (pyperformance port).
"""

import pyperf
from pprint import PrettyPrinter


printable = [('string', (1, 2), [3, 4], {5: 6, 7: 8})] * 100_000
p = PrettyPrinter()


if __name__ == '__main__':
    runner = pyperf.Runner()
    runner.metadata['description'] = 'pprint benchmark'

    if hasattr(p, '_safe_repr'):
        runner.bench_func('pprint_safe_repr', p._safe_repr,
                          printable, {}, None, 0)
    runner.bench_func('pprint_pformat', p.pformat, printable)



__all__ = [  # noqa: RUF022
    # filesystem
    "check_dir",
    "check_file",
    "temporary_file",
    # platform
    "MS_WINDOWS",
    # misc
    "check_name",
    "iter_clean_lines",
    "parse_name_pattern",
    "parse_selections",
    "parse_tag_pattern",
]


#######################################
# filesystem utils

import contextlib
import errno
import os
import os.path
import shlex
import shutil
import subprocess
import sys
import tempfile
import urllib.request


@contextlib.contextmanager
def temporary_file():
    tmp_filename = tempfile.mktemp()
    try:
        yield tmp_filename
    finally:
        try:
            os.unlink(tmp_filename)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise


def check_file(filename):
    if not os.path.isabs(filename):
        raise ValueError(f"expected absolute path, got {filename!r}")
    if not os.path.isfile(filename):
        raise ValueError(f"file missing ({filename})")


def check_dir(dirname):
    if not os.path.isabs(dirname):
        raise ValueError(f"expected absolute path, got {dirname!r}")
    if not os.path.isdir(dirname):
        raise ValueError(f"directory missing ({dirname})")


def resolve_file(filename, relroot=None):
    resolved = os.path.normpath(filename)
    resolved = os.path.expanduser(resolved)
    # resolved = os.path.expandvars(filename)
    if not os.path.isabs(resolved):
        if not relroot:
            relroot = os.getcwd()
        elif not os.path.isabs(relroot):
            raise NotImplementedError(relroot)
        resolved = os.path.join(relroot, resolved)
    return resolved


def safe_rmtree(path):
    if not os.path.exists(path):
        return False

    print("Remove directory %s" % path)
    # XXX Pass onerror to report on any files that could not be deleted?
    shutil.rmtree(path)
    return True


#######################################
# platform utils


MS_WINDOWS = sys.platform == "win32"


def run_cmd(argv, *, env=None, capture=None, verbose=True):
    try:
        cmdstr = " ".join(shlex.quote(a) for a in argv)
    except TypeError:
        print(argv)
        raise  # re-raise

    if capture is True:
        capture = "both"
    kw = dict(
        env=env,
    )
    if capture == "both":
        kw.update(
            dict(
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
        )
    elif capture == "combined":
        kw.update(
            dict(
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
            )
        )
    elif capture == "stdout":
        kw.update(
            dict(
                stdout=subprocess.PIPE,
            )
        )
    elif capture == "stderr":
        kw.update(
            dict(
                stderr=subprocess.PIPE,
            )
        )
    elif capture:
        raise NotImplementedError(repr(capture))
    if capture:
        kw.update(
            dict(
                encoding="utf-8",
            )
        )

    # XXX Use a logger.
    if verbose:
        print("#", cmdstr)

    # Explicitly flush standard streams, required if streams are buffered
    # (not TTY) to write lines in the expected order
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        proc = subprocess.run(argv, **kw)
    except OSError as exc:
        if exc.errno == errno.ENOENT:
            if verbose:
                print("command failed (not found)")
            return 127, None, None
        raise
    if proc.returncode != 0 and verbose:
        print(f"Command failed with exit code {proc.returncode}")
    return proc.returncode, proc.stdout, proc.stderr


def run_python(*args, python=sys.executable, **kwargs):
    if not isinstance(python, str) and python is not None:
        try:
            # See _pythoninfo.get_info().
            python = python.sys.executable
        except AttributeError:
            raise TypeError(f"expected python str, got {python!r}")
    return run_cmd([python, *args], **kwargs)


#######################################
# network utils


def download(url, filename):
    response = urllib.request.urlopen(url)
    with response:
        content = response.read()

    with open(filename, "wb") as fp:
        fp.write(content)
        fp.flush()


#######################################
# misc utils


def check_name(name, *, loose=False, allownumeric=False):
    if not name or not isinstance(name, str):
        raise ValueError(f"bad name {name!r}")
    if allownumeric:
        name = f"_{name}"
    if not loose:
        if name.startswith("-"):
            raise ValueError(name)
        if not name.replace("-", "_").isidentifier():
            raise ValueError(name)


def parse_name_pattern(text, *, fail=True):
    name = text
    # XXX Support globs and/or regexes?  (return a callable)
    try:
        check_name("_" + name)
    except Exception:
        if fail:
            raise  # re-raise
        return None
    return name


def parse_tag_pattern(text):
    if not text.startswith("<"):
        return None
    if not text.endswith(">"):
        return None
    tag = text[1:-1]
    # XXX Support globs and/or regexes?  (return a callable)
    check_name(tag)
    return tag


def parse_selections(selections, parse_entry=None):
    if isinstance(selections, str):
        selections = selections.split(",")
    if parse_entry is None:

        def parse_entry(o, e):
            return (o, e, None, e)

    for entry in selections:
        entry = entry.strip()
        if not entry:
            continue

        op = "+"
        if entry.startswith("-"):
            op = "-"
            entry = entry[1:]

        yield parse_entry(op, entry)


def iter_clean_lines(filename):
    with open(filename, encoding="utf-8") as reqsfile:
        for line in reqsfile:
            # strip comment
            line = line.partition("#")[0]
            line = line.rstrip()
            if not line:
                continue
            yield line



import pyperf

from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, sessionmaker
from sqlalchemy import create_engine


Base = declarative_base()


class Person(Base):
    __tablename__ = 'person'
    # Here we define columns for the table person
    # Notice that each column is also a normal Python instance attribute.
    id = Column(Integer, primary_key=True)
    name = Column(String(250), nullable=False)


class Address(Base):
    __tablename__ = 'address'
    # Here we define columns for the table address.
    # Notice that each column is also a normal Python instance attribute.
    id = Column(Integer, primary_key=True)
    street_name = Column(String(250))
    street_number = Column(String(250))
    post_code = Column(String(250), nullable=False)
    person_id = Column(Integer, ForeignKey('person.id'))
    person = relationship(Person)


# Create an engine that stores data in the local directory's
# sqlalchemy_example.db file.
engine = create_engine('sqlite://')

# Create all tables in the engine. This is equivalent to "Create Table"
# statements in raw SQL.
Base.metadata.create_all(engine)


# Bind the engine to the metadata of the Base class so that the
# declaratives can be accessed through a DBSession instance
Base.metadata.bind = engine

DBSession = sessionmaker(bind=engine)
# A DBSession() instance establishes all conversations with the database
# and represents a "staging zone" for all the objects loaded into the
# database session object. Any change made against the objects in the
# session won't be persisted into the database until you call
# session.commit(). If you're not happy about the changes, you can
# revert all of them back to the last commit by calling
# session.rollback()
session = DBSession()


# add 'npeople' people to the database

def bench_sqlalchemy(loops, npeople):
    total_dt = 0.0

    for loops in range(loops):
        # drop rows created by the previous benchmark
        session.query(Person).delete(synchronize_session=False)
        session.query(Address).delete(synchronize_session=False)

        # Run the benchmark once
        t0 = pyperf.perf_counter()

        for i in range(npeople):
            # Insert a Person in the person table
            new_person = Person(name="name %i" % i)
            session.add(new_person)
            session.commit()

            # Insert an Address in the address table
            new_address = Address(post_code='%05i' % i, person=new_person)
            session.add(new_address)
            session.commit()

        # do 100 queries per insert
        for i in range(npeople):
            session.query(Person).all()

        total_dt += (pyperf.perf_counter() - t0)

    return total_dt


def add_cmdline_args(cmd, args):
    cmd.extend(("--rows", str(args.rows)))


if __name__ == "__main__":
    runner = pyperf.Runner(add_cmdline_args=add_cmdline_args)
    runner.metadata['description'] = ("SQLAlchemy Declarative benchmark "
                                      "using SQLite")
    runner.argparser.add_argument("--rows", type=int, default=100,
                                  help="Number of rows (default: 100)")

    args = runner.parse_args()
    runner.bench_time_func('sqlalchemy_declarative', bench_sqlalchemy,
                           args.rows)



import importlib.util
import os
import sys
import sysconfig
import unittest

from pyperformance import _pythoninfo, tests

IS_VENV = sys.prefix != sys.base_prefix
CURRENT = {
    "executable (sys)": sys.executable,
    "executable (sys;realpath)": os.path.realpath(sys.executable),
    "base_executable": sys.executable,
    "base_executable (sys)": getattr(sys, "_base_executable", None),
    "version_str (sys)": sys.version,
    "version_info (sys)": sys.version_info,
    "hexversion (sys)": sys.hexversion,
    "api_version (sys)": sys.api_version,
    "pyc_magic_number": importlib.util.MAGIC_NUMBER,
    "implementation_name (sys)": sys.implementation.name,
    "implementation_version (sys)": sys.implementation.version,
    "platform (sys)": sys.platform,
    "prefix (sys)": sys.prefix,
    "exec_prefix (sys)": sys.exec_prefix,
    "base_prefix (sys)": sys.base_prefix,
    "base_exec_prefix (sys)": sys.base_exec_prefix,
    "stdlib_dir": os.path.dirname(os.__file__),
    "stdlib_dir (sys)": getattr(sys, "_stdlib_dir", None),
    "stdlib_dir (sysconfig)": sysconfig.get_path("stdlib"),
    "is_dev (sysconfig)": sysconfig.is_python_build(),
    "is_venv": sys.prefix != sys.base_prefix,
}
if IS_VENV:
    if CURRENT["base_executable"] == sys.executable:
        if CURRENT["base_executable (sys)"] == sys.executable:
            CURRENT["base_executable"] = None
        else:
            CURRENT["base_executable"] = CURRENT["base_executable (sys)"]


class GetInfoTests(tests.Functional, unittest.TestCase):
    maxDiff = 80 * 100

    def test_no_args(self):
        expected = _pythoninfo._build_info(CURRENT)

        info = _pythoninfo.get_info()

        self.assertEqual(vars(info), vars(expected))

    def test_current(self):
        expected = _pythoninfo._build_info(CURRENT)

        info = _pythoninfo.get_info(sys.executable)

        self.assertEqual(vars(info), vars(expected))

    def test_venv(self):
        expected = _pythoninfo._build_info(CURRENT)
        if IS_VENV:
            python = sys.executable
        else:
            venv, python, cleanup = tests.create_venv()
            self.addCleanup(cleanup)
            expected.sys.executable = python
            realpath = os.path.realpath(os.path.normpath(sys.executable))
            if os.name == "nt":
                # It isn't a symlink.
                expected.executable_realpath = os.path.realpath(python)
            expected.sys._base_executable = realpath
            expected.base_executable = realpath
            expected.sys.prefix = venv
            expected.sys.exec_prefix = venv
            expected.sys.version_info = tuple(expected.sys.version_info)
            (expected.sys.implementation.version) = tuple(
                expected.sys.implementation.version
            )
            expected.is_venv = True

        info = _pythoninfo.get_info(python)

        # We have to work around a possible bug.
        if not info.base_executable:
            expected.base_executable = None
            expected.sys._base_executable = info.sys._base_executable
        self.assertEqual(vars(info), vars(expected))



"""
Benchmark coverage performance with a recursive fibonacci function.
"""

import coverage
import pyperf


def fibonacci(n: int) -> int:
    if n <= 1:
        return n
    return fibonacci(n - 1) + fibonacci(n - 2)


def bench_coverage(loops: int) -> None:
    range_it = range(loops)
    cov = coverage.Coverage()
    cov.start()
    t0 = pyperf.perf_counter()
    for _ in range_it:
        fibonacci(25)
    cov.stop()
    return pyperf.perf_counter() - t0


if __name__ == "__main__":
    runner = pyperf.Runner()
    runner.metadata['description'] = "Benchmark coverage"
    runner.bench_time_func('coverage', bench_coverage)
